#include <stdio.h>
#include <string.h>
#include <glib.h>
#include "obtheme.h"

static void decor_print(gpointer data, gpointer user_data)
{
	struct decor *decor = data;
	printf("    decor id %s\n", decor->name);
	printf("      anchor (%d %d %d)\n", decor->space.anchor.x, decor->space.anchor.y, decor->space.anchor.z);
	printf("      up     (%d %d %d)\n", decor->space.up.x, decor->space.up.y, decor->space.up.z);
	if (decor->children)
		g_slist_foreach(decor->children, decor_print, NULL);
}

static void style_print(gpointer key, gpointer value, gpointer user_data)
{
	char *stylename = key;
	struct style *style = value;

	printf("  style %s\n", stylename);
	g_slist_foreach(style->tree, decor_print, NULL);
}

static void theme_print(gpointer key, gpointer value, gpointer user_data)
{
	char *name = key;
	struct theme *thm = value;
	printf("name = %s\n", name);
	g_hash_table_foreach(thm->styles, style_print, NULL);
}

static void material_print(gpointer key, gpointer value, gpointer user_data)
{
	char *name = key;
	struct material *mat = value;
	printf("name = %s\n", name);
	printf(" opacity = %f\n", mat->opacity);
}

int main(int argc, char **argv)
{
	int err;
	struct obthemedata themedata;

	themedata.themes = g_hash_table_new(g_str_hash, g_str_equal);
	themedata.materials = g_hash_table_new(g_str_hash, g_str_equal);
	err = obtheme_parse(&themedata, argv[1]);
	printf("err = %d\n", err);

	g_hash_table_foreach(themedata.materials, material_print, NULL);

	g_hash_table_foreach(themedata.themes, theme_print, NULL);

	return 0;
}
